using System;
using UnityEngine;
using SimpleJSON;

namespace VamTastic
{
    public class Bunker : MVRScript
    {
        #region PluginInfo
        public string pluginAuthor = "VamTastic";
        public string pluginName = "Bunker Doors";
        public string pluginVersion = "1";
        public string pluginDate = "20/05/2023";
        public string pluginDescription = "Used to open and close the doors";
        #endregion

        #region Variables
        private JSONStorableFloat[] doorScrubbers = new JSONStorableFloat[6];
        private float[] previousValues = new float[6];
        private Transform[] doors = new Transform[6];
        
        private int initLoops = 0;
        private const int MaxInitLoops = 1500;

        private bool editorInitialized = false;

        public Transform ObjectRoot { get; private set; }
        #endregion

        public override void Init()
        {
            try
            {
                if (containingAtom.type != "CustomUnityAsset")
                {
                    SuperController.LogError("Bunker: Please add this script on a Custom Unity Asset (script disabled).");
                    return;
                }

                InitializeUI();
            }
            catch (Exception e)
            {
                SuperController.LogError($"Exception caught: {e}");
            }
        }

        private void InitializeUI()
        {
            for (int i = 0; i < 6; i++)
            {
                int index = i;  // Capture variable for closure
                doorScrubbers[i] = new JSONStorableFloat($"Door {i + 1}", 0f, value => OnSliderChanged(value, index), 0f, 1f);
                RegisterFloat(doorScrubbers[i]);
                CreateSlider(doorScrubbers[i], false);
            }

            JSONStorableString helpText = new JSONStorableString("About", "<color=#000><size=35><b>About</b></size></color>\n\n<color=#333>Credits to Hazmhox and Frief.\n\nBunker Doors.\nThe doors can be triggered using Acidbubbles Timeline to open and close.\nif you have a door open and save the scene the position of a door is also saved.\n</color>");
            UIDynamic helpTextfield = CreateTextField(helpText, true);
            helpTextfield.height = 1100.0f;
        }

        void Update()
        {
            if (initLoops < MaxInitLoops && !editorInitialized && !SuperController.singleton.isLoading)
            {
                initLoops++;
                editorInitialized = InitEditor();
            }

            if (initLoops == MaxInitLoops)
            {
                SuperController.LogError("Bunker: Initialization failed. Please select a compatible object and reload the script.");
                initLoops++;
            }
        }

        private bool InitEditor()
        {
            if (editorInitialized) return true;

            ObjectRoot = FindTransform(containingAtom.reParentObject, "Bunker");
            if (ObjectRoot == null) return false;

            for (int i = 0; i < 6; i++)
            {
                doors[i] = FindTransform(ObjectRoot, $"door0{i + 1}");
                if (doors[i] == null)
                {
                    SuperController.LogError("Bunker: Missing mandatory components.");
                    return false;
                }
            }

            return true;
        }

        private Transform FindTransform(Transform parent, string nameToFind)
        {
            foreach (Transform child in parent)
            {
                if (child.name.Contains(nameToFind)) return child;

                Transform result = FindTransform(child, nameToFind);
                if (result != null) return result;
            }
            return null;
        }

        private void OnSliderChanged(float value, int doorIndex)
        {
            if (doors[doorIndex] == null) return;

            float change = value - previousValues[doorIndex];
            doors[doorIndex].transform.Rotate(Vector3.back * change * 180);
            previousValues[doorIndex] = value;
        }
    }
}
